% La fonction "annale06_q2_v2" fait la même chose que "annale06_q2_v1",
% mais avec cinq variables de contrôle qui sont, dans l'ordre, |y|, y^2, x,
% |x| et x^2. Je ne commente que les points qui diffèrent. Notez qu'ici
% j'ai traité la liste des différentes variables de façon vectorielle, ce
% qui rend le code paradoxalement plus compact.
function annale06_q2_v2(N)
T = [1,0];
% Le 6-vecteur "S" qui contiendra les sommes respectives de f et des cinq
% variables de contrôle.
S = zeros(1,6);
% La 6×6-matrice "SP" contiendra les sommes respectives des produits entre
% les différentes variables (càd. f et les 5 variables de contrôle).
SP = zeros(6);
for i = 1:N
    X = randn(1,2);
    % Le 6-vecteur "fetgs" est la valeur de la simulation de f et des cinq
    % variables de contrôle.
    fetgs = [sqrt(sum((T-X).^2)),abs(X(2)),X(2)^2,X(1),abs(X(1)),X(1)^2];
    % On met à jour "S" et "SP".
    S = S + fetgs;
    SP = SP + fetgs' * fetgs;
end
% Le 6-vecteur "m" est la liste des moyennes empiriques de f (moyenne qui
% correspond à "m(1)") et des g (moyennes qui correspondent à "m(2:6)").
m = S / N;
% La 6×6-matrice "Var" est la matrice de covariance (empirique) de f et des
% g. Celle-ci contient donc à la fois la variance de f (qui sera
% "Var(1,1)"), la matrice de covariance de g (qui sera "Var(2:6,2:6)") et
% la liste des covariances entre f et les différentes g (qui sera
% "Var(1,2:6)").
Var = SP / N - m' * m;
% Le 5-vecteur "lambda" est (l'évaluation empirique de) le lambda optimal.
% Attention, l'opération '/' désigne ici une divison de matrices : voir
% "doc mrdivide" pour plus de détails.
lambda = Var(1,2:6) / Var(2:6,2:6);
% "esperance" est l'estimateur de E(f-sum(lambda_k*g_k)). Notez que "lambda
% * m(2:6)'" aurait pu être écrit plus explicitement comme "sum(lambda .*
% m(2:6))", mais ce n'aurait pas été pertinent dans ce code où tout est
% traité vectoriellement.
esperance = m(1) - lambda * m(2:6)';
% "variance" est la variance (empirique) de f-sum(lambda_k*g_k).
variance = Var(1,1) - 2 * lambda * Var(1,2:6)' + lambda * Var(2:6,2:6) * lambda';
% "erreur" est la marge d'erreur sur l'estimateur de
% E(f-sum(lambda_k*g_k)), qui sera aussi celle sur l'estimateur de E(f) une
% fois qu'on ajoura ajouté sum(lambda_k*E(g_k)).
erreur = 2 * sqrt(variance) / sqrt(N);
% Le 5-vecteur "Eg" est la liste des espérances *exactes* des g, dont on a
% calculé les expressions théoriquement.
Eg = [2 / sqrt(2*pi), 1, 0, 2 / sqrt(2 * pi), 1];
% "D" est l'estimateur de E(f) utilisant la variable de contrôle.
D = esperance + lambda * Eg';
disp(['Distance moyenne : ',num2str(D),' ± ',num2str(erreur)]);
disp(['Pour info : les lambda_k optimaux valaient environ ',num2str(lambda)]);
return
end